import { inject, Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { BehaviorSubject, catchError, EMPTY, Observable, tap } from 'rxjs';
import { User } from '../model/User';
import { environment } from '@environments/environment';

@Injectable({
  providedIn: 'root',
})
export class UserService {
  private apiUrl = `${environment.apiUrl}/users`;

  private currentUserSubject = new BehaviorSubject<User | null>(null);
  private http: HttpClient = inject(HttpClient);


  public getCurrentUser(): Observable<User | null> {
    return this.http.get<User | null>('/backend/user').pipe(
      catchError(() => EMPTY),
      tap((user) => this.currentUserSubject.next(user))
    );
  }

  public refreshCurrentUser(): void {
    this.getCurrentUser().subscribe();
  }

  public updateLocalBalance(amount: number): void {
    const currentUser = this.currentUserSubject.getValue();
    if (currentUser) {
      const updatedUser = {
        ...currentUser,
        balance: currentUser.balance + amount,
      };
      this.currentUserSubject.next(updatedUser);
    }
  }
}
